@@ -7,6 +7,7 @@
 from .algebraic.slim import SLIMRegressor, SLIMClassifier
 from .discretization.discretizer import RFDiscretizer, BasicDiscretizer
 from .discretization.mdlp import MDLPDiscretizer, BRLDiscretizer
+from .experimental.bartpy import BART
 from .rule_list.bayesian_rule_list.bayesian_rule_list import BayesianRuleListClassifier
 from .rule_list.corels_wrapper import OptimalRuleListClassifier
 from .rule_list.greedy_rule_list import GreedyRuleListClassifier
@@ -26,18 +27,20 @@ from .tree.cart_ccp import DecisionTreeCCPClassifier, DecisionTreeCCPRegressor,
 # from .tree.iterative_random_forest.iterative_random_forest import IRFClassifier
 # from .tree.optimal_classification_tree import OptimalTreeModel
 from .tree.cart_wrapper import GreedyTreeClassifier, GreedyTreeRegressor
-from .tree.figs import FIGSRegressor, FIGSClassifier
+from .tree.figs import FIGSRegressor, FIGSClassifier, FIGSRegressorCV, FIGSClassifierCV
 from .tree.gosdt.pygosdt import OptimalTreeClassifier
 from .tree.gosdt.pygosdt_shrinkage import HSOptimalTreeClassifier, HSOptimalTreeClassifierCV
 from .tree.hierarchical_shrinkage import HSTreeRegressor, HSTreeClassifier, HSTreeRegressorCV, HSTreeClassifierCV
+from .tree.tao import TaoTreeClassifier, TaoTreeRegressor
+from .util.data_util import get_clean_dataset
 from .util.distillation import DistilledRegressor
 from .util.explain_errors import explain_classification_errors
-from .util.data_util import get_clean_dataset
 
 CLASSIFIERS = [BayesianRuleListClassifier, GreedyRuleListClassifier, SkopeRulesClassifier,
                BoostedRulesClassifier, SLIMClassifier, SlipperClassifier, BayesianRuleSetClassifier,
                C45TreeClassifier, OptimalTreeClassifier, OptimalRuleListClassifier, OneRClassifier,
-               SlipperClassifier,
+               SlipperClassifier, RuleFitClassifier, TaoTreeClassifier,
                FIGSClassifier, HSTreeClassifier, HSTreeClassifierCV]  # , IRFClassifier
-REGRESSORS = [RuleFitRegressor, SLIMRegressor, GreedyTreeClassifier, FIGSRegressor, HSTreeRegressor, HSTreeRegressorCV]
+REGRESSORS = [RuleFitRegressor, SLIMRegressor, GreedyTreeClassifier, FIGSRegressor,
+              TaoTreeRegressor, HSTreeRegressor, HSTreeRegressorCV, BART]
 DISCRETIZERS = [RFDiscretizer, BasicDiscretizer, MDLPDiscretizer, BRLDiscretizer]
