@@ -1,4 +1 @@
-"""
-.. include:: readme.md
-"""
 from .sklearnmodel import BART, ShrunkBART, ShrunkBARTCV
\ No newline at end of file
